#include "fmtfun.hh"
#include "colors.hh"

const std::string Formatting::GetUpdatedCodes()
{

    /* Translates the current MXP formattings into
     * Xterm-256color compatible escape codes. */ 
    std::vector<unsigned> defs;
    
    static bool was_fore = true;
    static bool was_back = true;
    static bool was_B = false;
    static bool was_U = false;
    
    unsigned forec = 0xC0C0C0; bool foreset=false;
    unsigned backc = 0x000000; bool backset=false;
    
    for(std::vector<FontDef>::const_iterator
        i = FontDefs.begin();
        i != FontDefs.end();
        ++i)
    {
        if(!i->forecolordefs.empty())
            { foreset=true; forec = TranslateColorNameToRGB(i->forecolordefs); }
        if(!i->backcolordefs.empty())
            { backset=true; backc = TranslateColorNameToRGB(i->backcolordefs); }
    }
    
    #define addm(n) defs.push_back((n))
    
    /* xterm-256color escapes:
      SET:   1=bold 4=underscore 5=blink 7=reverse 8=invisible
      UNSET: 2x  with the same as above
      
      FORE:  SET: 3x {x=0..7}, 38;5;x {x=0..255}
             UNSET: 39
      BACK:  Same except 4 instead of 3
      
      0 resets colors and flags.
    */
    
    if(Bright) { if(foreset) forec |= 0xC0C0C0; }

    if(foreset)
    {
        const unsigned xtermcolor = CreateXtermColor(forec);
        if(xtermcolor < 8)
            addm(30 + xtermcolor);
        else
            { addm(38); addm(5); addm(xtermcolor); }
        was_fore = true;
    }
    else if(was_fore)
        { was_fore=false; addm(39); }

    if(backset)
    {
        const unsigned xtermcolor = CreateXtermColor(backc);
        if(xtermcolor < 8)
            addm(40 + xtermcolor);
        else
            { addm(48); addm(5); addm(xtermcolor); }
        was_back=true;
    }
    else if(was_back)
        { was_back=false; addm(49); }

    if(B) { if(!was_B) { was_B=true; addm(1); } } else if(was_B) { was_B=false; addm(22); }
    if(I) {}
    if(U) { if(!was_U) { was_U=true; addm(4); } } else if(was_U) { was_U=false; addm(24); }
    if(S) {}
    
    #undef addm

    std::string result = "\33[";
    for(unsigned a=0; a<defs.size(); ++a)
    {
        char Buf[32];
        sprintf(Buf, "%s%u", a?";":"", defs[a]);
        result += Buf;
    }
    result += 'm';
    return result;
}
